﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using RSN.Tools;

namespace RSN.DotP
{
	public partial class Options : Form
	{
		private DeckBuilder m_frmMain = null;
		private IdScheme m_isScheme;
		private bool m_bLoading;

		public Options(DeckBuilder frmMain)
		{
			InitializeComponent();

			m_bLoading = true;

			m_frmMain = frmMain;

			// Id Scheme options need to be done before loading of strings to make sure we load the right strings into the scheme type drop down.
			m_isScheme = Settings.GetSerializableSetting("CurrentIdScheme", new IdScheme());
			cboSchemeType.DisplayMember = "Value";
			cboSchemeType.ValueMember = "Key";
			numMinDigits.Value = m_isScheme.MinimumIdDigits;
			numMinId.Value = m_isScheme.MinimumId;
			numMaxId.Value = m_isScheme.MaximumId;
			numDeckChange.Value = m_isScheme.DeckIdChange;
			numLandPoolChange.Value = m_isScheme.LandPoolIdChange;
			numRegularUnlockChange.Value = m_isScheme.RegularUnlockIdChange;
			numPromoUnlockChange.Value = m_isScheme.PromoUnlockIdChange;

			LoadLocalizedStrings();

			Rectangle rcPosition = Settings.GetSetting("OptionsPosition", new Rectangle(-1, -1, -1, -1));
			if (rcPosition.X != -1)
			{
				bool bDoChecks = Settings.GetSetting("PerformBasicScreenChecks", true);
				if ((rcPosition.X >= 0) && (rcPosition.Y >= 0) &&
					(rcPosition.X < (Screen.PrimaryScreen.WorkingArea.X + Screen.PrimaryScreen.WorkingArea.Width - 50)) &&
					(rcPosition.Y < (Screen.PrimaryScreen.WorkingArea.Y + Screen.PrimaryScreen.WorkingArea.Height - 50)) || !bDoChecks)
				{
					this.StartPosition = FormStartPosition.Manual;
					this.Location = rcPosition.Location;
				}
				// This is a non-resizable form.
				//this.Size = rcPosition.Size;
			}

			// I have form sizing issues if I try to set this in the designer when coupled with a manual start position rather than from code.
			this.FormBorderStyle = FormBorderStyle.FixedDialog;

			txtGameDir.Text = Settings.GetSetting("DotP2013Directory", string.Empty);

			// Fill the languages list.
			foreach (KeyValuePair<string, string> lang in Settings.Languages)
				cboLanguage.Items.Add(lang);
			cboLanguage.DisplayMember = "Value";
			cboLanguage.ValueMember = "Key";
			if (Settings.Languages.ContainsKey(Settings.LanguageCode))
				cboLanguage.SelectedItem = new KeyValuePair<string, string>(Settings.LanguageCode, Settings.Languages[Settings.LanguageCode]);

			chkBasicScreenChecks.Checked = Settings.GetSetting("PerformBasicScreenChecks", true);

			if (txtGameDir.Text.Length == 0)
				cmdCancel.Enabled = false;
			else
				cmdCancel.Enabled = true;

			m_bLoading = false;

			// Initialize the Id Scheme Example
			numChosenId_ValueChanged(null, null);
		}

		private void LoadLocalizedStrings(string strLangCode = "")
		{
			LanguageStrings lsStrings = Settings.UIStrings;
			if (strLangCode.Length > 0)
			{
				try
				{
					LanguageStrings lsLoad = new LanguageStrings(strLangCode);
					lsStrings = lsLoad;
				}
				catch (Exception e)
				{
					Settings.ReportError(e, ErrorPriority.Medium, "Unable to load Language Strings for language code: " + strLangCode);
				}
			}

			// I could simply iterate over all of the controls in this form and replace
			//	their text with localized strings using the tags as a key, but then I
			//	might forget to resize certain other controls when their label counterpart
			//	changes.
			this.Text = lsStrings[(string)this.Tag];
			gbGeneral.Text = lsStrings[(string)gbGeneral.Tag];
			lblGameDir.Text = lsStrings[(string)lblGameDir.Tag];
			cmdBrowse.Text = lsStrings[(string)cmdBrowse.Tag];
			lblLanguage.Text = lsStrings[(string)lblLanguage.Tag];
			Tools.ResizeRelatedControl(cboLanguage, lblLanguage);
			chkBasicScreenChecks.Text = lsStrings[(string)chkBasicScreenChecks.Tag];
			cmdApply.Text = lsStrings[(string)cmdApply.Tag];
			cmdCancel.Text = lsStrings[(string)cmdCancel.Tag];

			// For Id Scheme stuff
			gbIdScheme.Text = lsStrings[(string)gbIdScheme.Tag];
			lblSchemeType.Text = lsStrings[(string)lblSchemeType.Tag];
			lblMinDigits.Text = lsStrings[(string)lblMinDigits.Tag];
			Tools.ResizeRelatedControl(numMinDigits, lblMinDigits);
			lblMinId.Text = lsStrings[(string)lblMinId.Tag];
			Tools.ResizeRelatedControl(numMinId, lblMinId);
			lblMaxId.Text = lsStrings[(string)lblMaxId.Tag];
			Tools.ResizeRelatedControl(numMaxId, lblMaxId);
			lblDeckChange.Text = lsStrings[(string)lblDeckChange.Tag];
			Tools.ResizeRelatedControl(numDeckChange, lblDeckChange);
			lblLandPoolChange.Text = lsStrings[(string)lblLandPoolChange.Tag];
			Tools.ResizeRelatedControl(numLandPoolChange, lblLandPoolChange);
			lblRegularUnlockChange.Text = lsStrings[(string)lblRegularUnlockChange.Tag];
			Tools.ResizeRelatedControl(numRegularUnlockChange, lblRegularUnlockChange);
			lblPromoUnlockChange.Text = lsStrings[(string)lblPromoUnlockChange.Tag];
			Tools.ResizeRelatedControl(numPromoUnlockChange, lblPromoUnlockChange);
			lblIdExample.Text = lsStrings[(string)lblIdExample.Tag];

			// To load the Scheme Types I have to reload the combo box
			cboSchemeType.Items.Clear();
			cboSchemeType.Items.Add(new KeyValuePair<IdSchemeType, string>(IdSchemeType.AdditiveScheme, lsStrings[IdSchemeType.AdditiveScheme.ToString().ToUpper()]));
			cboSchemeType.Items.Add(new KeyValuePair<IdSchemeType, string>(IdSchemeType.PrefixScheme, lsStrings[IdSchemeType.PrefixScheme.ToString().ToUpper()]));
			cboSchemeType.Items.Add(new KeyValuePair<IdSchemeType, string>(IdSchemeType.SuffixScheme, lsStrings[IdSchemeType.SuffixScheme.ToString().ToUpper()]));

			// For combo boxes that need to be re-selected after string change.
			if (m_bLoading)
				cboSchemeType.SelectedItem = new KeyValuePair<IdSchemeType, string>(m_isScheme.Type, lsStrings[m_isScheme.Type.ToString().ToUpper()]);
			else
			{
				m_bLoading = true;
				cboSchemeType.SelectedItem = new KeyValuePair<IdSchemeType, string>(m_isScheme.Type, lsStrings[m_isScheme.Type.ToString().ToUpper()]);
				m_bLoading = false;
			}
		}

		private void cboLanguage_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				if (cboLanguage.SelectedIndex > -1)
				{
					LoadLocalizedStrings(((KeyValuePair<string, string>)cboLanguage.SelectedItem).Key);
				}
			}
		}

		private void cmdCancel_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void cmdApply_Click(object sender, EventArgs e)
		{
			if ((txtGameDir.Text.Trim().Length == 0) || (!Directory.Exists(txtGameDir.Text.Trim())))
			{
				// We have a problem: we don't have a valid directory.
				MessageBox.Show(Settings.UIStrings["NEED_VALID_GAME_DRIECTORY_MESSAGE"], Settings.UIStrings["NEED_VALID_GAME_DRIECTORY_CAPTION"], MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

			Settings.SaveSetting("DotP2013Directory", txtGameDir.Text.Trim());
			Settings.ChangeLanguage(((KeyValuePair<string, string>)cboLanguage.SelectedItem).Key);
			m_frmMain.LoadLocalizedStrings();
			Settings.SaveSetting("PerformBasicScreenChecks", chkBasicScreenChecks.Checked);

			// Id Scheme Options
			Settings.SaveSerializableSetting("CurrentIdScheme", m_isScheme);

			this.Close();
		}

		private void cmdBrowse_Click(object sender, EventArgs e)
		{
			OpenFileDialog ofdChoose = new OpenFileDialog();
			ofdChoose.AutoUpgradeEnabled = true;
			ofdChoose.CheckFileExists = true;
			ofdChoose.CheckPathExists = true;
			ofdChoose.DereferenceLinks = true;
			ofdChoose.Filter = Settings.UIStrings["CHOOSE_DIRECTORY_FILTER"];
			ofdChoose.FilterIndex = 1;
			ofdChoose.Title = Settings.UIStrings["CHOOSE_DIRECTORY_TITLE"];
			if (txtGameDir.Text.Trim().Length > 0)
				ofdChoose.InitialDirectory = txtGameDir.Text.Trim();
			ofdChoose.FileName = "DotP_D13.exe";
			DialogResult drResult = ofdChoose.ShowDialog();
			if (drResult == DialogResult.OK)
			{
				txtGameDir.Text = Path.GetDirectoryName(ofdChoose.FileName);
			}
		}

		private void Options_FormClosing(object sender, FormClosingEventArgs e)
		{
			Settings.SaveSetting("OptionsPosition", new Rectangle(this.Location, this.Size));
		}

		private void numChosenId_ValueChanged(object sender, EventArgs e)
		{
			try
			{
				lblDeckId.Text = m_isScheme.GetDeckId((int)numChosenId.Value).ToString();
			}
			catch(Exception)
			{
				lblDeckId.Text = Settings.UIStrings["INVALID"];
			}
			try
			{
				lblLandPoolId.Text = m_isScheme.GetLandPoolId((int)numChosenId.Value).ToString();
			}
			catch (Exception)
			{
				lblLandPoolId.Text = Settings.UIStrings["INVALID"];
			}
			try
			{
				lblRegularUnlockId.Text = m_isScheme.GetRegularUnlockId((int)numChosenId.Value).ToString();
			}
			catch (Exception)
			{
				lblRegularUnlockId.Text = Settings.UIStrings["INVALID"];
			}
			try
			{
				lblPromoUnlockId.Text = m_isScheme.GetPromoUnlockId((int)numChosenId.Value).ToString();
			}
			catch (Exception)
			{
				lblPromoUnlockId.Text = Settings.UIStrings["INVALID"];
			}
		}

		private void cboSchemeType_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				m_isScheme.Type = ((KeyValuePair<IdSchemeType, string>)cboSchemeType.SelectedItem).Key;
				numChosenId_ValueChanged(null, null);
			}
			switch (m_isScheme.Type)
			{
				case IdSchemeType.AdditiveScheme:
					lblMinDigits.Enabled = false;
					numMinDigits.Enabled = false;
					break;
				case IdSchemeType.PrefixScheme:
					lblMinDigits.Enabled = true;
					numMinDigits.Enabled = true;
					break;
				case IdSchemeType.SuffixScheme:
					lblMinDigits.Enabled = false;
					numMinDigits.Enabled = false;
					break;
			}
		}

		private void numMinDigits_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				m_isScheme.MinimumIdDigits = (int)numMinDigits.Value;
				numChosenId_ValueChanged(null, null);
			}
		}

		private void numMinId_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				try
				{
					m_isScheme.MinimumId = (int)numMinId.Value;
					numChosenId.Minimum = m_isScheme.MinimumId;
					numChosenId_ValueChanged(null, null);
				}
				catch (Exception)
				{
					numMinId.Value = m_isScheme.MinimumId;
				}
			}
		}

		private void numMaxId_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				try
				{
					m_isScheme.MaximumId = (int)numMaxId.Value;
					numChosenId.Maximum = m_isScheme.MaximumId;
					numChosenId_ValueChanged(null, null);
				}
				catch (Exception)
				{
					numMaxId.Value = m_isScheme.MaximumId;
				}
			}
		}

		private void numDeckChange_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				m_isScheme.DeckIdChange = (int)numDeckChange.Value;
				numChosenId_ValueChanged(null, null);
			}
		}

		private void numLandPoolChange_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				m_isScheme.LandPoolIdChange = (int)numLandPoolChange.Value;
				numChosenId_ValueChanged(null, null);
			}
		}

		private void numRegularUnlockChange_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				m_isScheme.RegularUnlockIdChange = (int)numRegularUnlockChange.Value;
				numChosenId_ValueChanged(null, null);
			}
		}

		private void numPromoUnlockChange_ValueChanged(object sender, EventArgs e)
		{
			if (!m_bLoading)
			{
				m_isScheme.PromoUnlockIdChange = (int)numPromoUnlockChange.Value;
				numChosenId_ValueChanged(null, null);
			}
		}
	}
}
